<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:lav="http://software.in2p3.fr/lavoisier/application.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/application.xsd"
                exclude-result-prefixes="xalan lav">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="lav:*"/>
    <xsl:param name="_file_"/>
    <xsl:param name="_authenticators_"/>

    <xsl:template match="/lav:views | /lav:processors | /lav:authenticators">
        <xsl:copy><xsl:copy-of select="@*"/>
            <xsl:apply-templates>
                <xsl:with-param name="file" select="$_file_"/>
                <xsl:with-param name="category">
                    <xsl:choose>
                        <xsl:when test="$_file_='engine/post-processors.xml'"/>
                        <xsl:when test="$_file_='service/authenticators.xml'"/>
                        <xsl:when test="$_file_='app/views.xml'">Test</xsl:when>
                        <xsl:when test="starts-with($_file_,'lib/')">Hidden</xsl:when>
                        <xsl:when test="contains($_file_,'/views/')">
                            <xsl:value-of select="substring-before(substring-after($_file_,'/views/'),'.xml')"/>
                        </xsl:when>
                        <xsl:when test="contains($_file_,'/views.xml')">Test</xsl:when>
                        <xsl:otherwise>
                            <xsl:message terminate="yes">Unexpected file name: <xsl:value-of select="$_file_"/></xsl:message>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:with-param>
                <xsl:with-param name="authenticators" select="$_authenticators_"/>
            </xsl:apply-templates>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="lav:view">
        <xsl:param name="file"/>
        <xsl:param name="category"/>
        <xsl:param name="authenticators"/>
        <xsl:copy><xsl:copy-of select="@*"/>
            <!-- set @_file_ -->
            <xsl:if test="@_file_">
                <xsl:message terminate="yes">Attribute @_file_ is reserved for internal needs, please do not set it!</xsl:message>
            </xsl:if>
            <xsl:attribute name="_file_"><xsl:value-of select="$file"/></xsl:attribute>

            <!-- set @authenticators (if not already defined) -->
            <xsl:if test="not(@authenticators) and $authenticators">
                <xsl:attribute name="authenticators"><xsl:value-of select="$authenticators"/></xsl:attribute>
            </xsl:if>

            <!-- add <namespace> -->
            <xsl:call-template name="NAMESPACE"/>
            <xsl:apply-templates select="lav:namespace"/>

            <!-- set <category> -->
            <xsl:if test="not(lav:info)">
                <info>
                    <category>
                        <xsl:call-template name="CATEGORY">
                            <xsl:with-param name="category" select="$category"/>
                        </xsl:call-template>
                    </category>
                </info>
            </xsl:if>

            <!-- recurse -->
            <xsl:apply-templates select="*[not(self::lav:namespace)] | text() | comment()">
                <xsl:with-param name="category" select="$category"/>
            </xsl:apply-templates>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="lav:pre-renderers">
        <xsl:copy><xsl:copy-of select="@*"/>
            <!-- add <namespace> -->
            <xsl:call-template name="NAMESPACE"/>

            <!-- recurse -->
            <xsl:apply-templates select="* | text() | comment()"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="lav:info">
        <xsl:param name="category"/>
        <xsl:copy><xsl:copy-of select="@*"/>
            <!-- set <category> -->
            <xsl:if test="lav:category">
                <xsl:message terminate="yes">Category must be set in attribute /views/@name rather than in element &lt;category&gt;</xsl:message>
            </xsl:if>
            <category>
                <xsl:call-template name="CATEGORY">
                    <xsl:with-param name="category" select="$category"/>
                </xsl:call-template>
            </category>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <xsl:template name="CATEGORY">
        <xsl:param name="category"/>
        <xsl:choose>
            <xsl:when test="ancestor::lav:views/@name"><xsl:value-of select="ancestor::lav:views/@name"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="$category"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="NAMESPACE">
        <!-- convert xmlns:ns to <namespace prefix="ns"/> -->
        <!-- WARNING: running this from DOM makes order of <namespace> unpredictable -->
        <xsl:for-each select="(/* | ancestor::* | descendant-or-self::*)/namespace::*
                                    [name()!='']
                                    [.!='http://www.w3.org/XML/1998/namespace']">
            <namespace prefix="{name()}" uri="{.}"/>
        </xsl:for-each>
    </xsl:template>

    <!-- default rules -->
    <xsl:template match="comment()">
        <xsl:comment><xsl:value-of select="."/></xsl:comment>
    </xsl:template>
    <xsl:template match="*">
        <xsl:copy><xsl:copy-of select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>