<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:lav="http://software.in2p3.fr/lavoisier/application.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/application.xsd"
                exclude-result-prefixes="xalan lav">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="lav:*"/>

    <!-- checks -->
    <xsl:template match="/">
        <xsl:for-each select="descendant::lav:view[lav:cache]/lav:argument">
            <xsl:for-each select="following-sibling::lav:*/descendant-or-self::lav:*
                                  [not(self::lav:argument | self::lav:variable | ancestor::lav:cache | ancestor::lav:renderers)]
                                  /@*[contains(.,concat('$',current()/@name))]">
                <xsl:message terminate="yes">Error in view '<xsl:value-of select="parent::lav:view/@name"/>':
                    Detected incompatibility between cache and this XPath expression: <xsl:value-of select="."/>
                </xsl:message>
            </xsl:for-each>
        </xsl:for-each>
        <xsl:for-each select="descendant::lav:cache[lav:trigger[contains(@type,'ViewCreated')]][lav:trigger[contains(@type,'DependencyRefreshed')]]">
            <xsl:message terminate="yes">Warning on view '<xsl:value-of select="parent::lav:view/@name"/>:
                Cache uses incompatible trigger plugins (ViewCreated and DependencyRefreshed).
            </xsl:message>
        </xsl:for-each>
        <!-- start -->
        <xsl:apply-templates/>
    </xsl:template>

    <!-- add default post-processors and authenticators -->
    <xsl:template match="lav:processors-list">
        <xsl:copy><xsl:copy-of select="@*"/>
            <xsl:if test="not(lav:processors[not(@name) or @name='_post-view_'])">
                <processors/>
            </xsl:if>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="lav:authenticators-list">
        <xsl:copy><xsl:copy-of select="@*"/>
            <xsl:if test="not(lav:authenticators[not(@name) or @name='_authenticators_'])">
                <authenticators/>
            </xsl:if>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <!-- add default renderer -->
    <xsl:template match="lav:renderers[lav:renderer/@type='HTMLRenderer']">
        <xsl:copy>
            <xsl:apply-templates select="lav:renderer"/>
            <xsl:if test="not(lav:renderer[@type='DefaultRenderer'])">
                <renderer type="DefaultRenderer">
                    <xsl:apply-templates select="self::lav:renderers" mode="PARAMETER"/>
                </renderer>
            </xsl:if>
            <xsl:if test="not(lav:renderer[@type='PDFRenderer'])">
                <renderer type="PDFRenderer">
                    <xsl:apply-templates select="self::lav:renderers" mode="PARAMETER"/>
                </renderer>
            </xsl:if>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="lav:renderers[lav:renderer/@type='HTMLRenderer']/lav:renderer[(@type='DefaultRenderer' or @type='PDFRenderer') and not(lav:parameter[@name='template'])]">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates/>
            <xsl:apply-templates select="parent::lav:renderers" mode="PARAMETER"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="lav:renderers[lav:renderer/@type='HTMLRenderer']" mode="PARAMETER">
        <xsl:apply-templates select="lav:renderer[@type='HTMLRenderer']/lav:parameter[@name='template' or @name='parameters']"/>
    </xsl:template>

    <!-- add default cache parameter: "xpath" -->
    <xsl:template match="lav:cache[@type='FileCache' or @type='fr.in2p3.lavoisier.cache.FileCache' or
                                   @type='EmptyCache' or @type='fr.in2p3.lavoisier.cache.EmptyCache']" priority="666">
        <xsl:copy><xsl:apply-templates select="@*|*|text()|comment()"/></xsl:copy>
    </xsl:template>
    <xsl:template match="lav:cache[not(lav:parameter/@name='xpath')]">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="lav:parameter"/>
            <xsl:comment> generated automatically </xsl:comment>
            <parameter name="xpath" eval="path(false())"/>
            <xsl:apply-templates select="lav:trigger"/>
        </xsl:copy>
    </xsl:template>

    <!-- default rules -->
    <xsl:template match="@*">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="*">
        <xsl:copy>
            <xsl:apply-templates select="@*|*|text()|comment()"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>