<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:tpl="http://software.in2p3.fr/lavoisier/application.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/plan.xsd"
                exclude-result-prefixes="xalan tpl">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="*"/>

    <xsl:template match="tpl:element | tpl:element-ignore">
        <e><xsl:copy-of select="namespace::*[. != 'http://software.in2p3.fr/lavoisier/application.xsd']"/>
            <xsl:copy-of select="@in | @out | @out-eval | @if | @if-on-future | @future | @attributes | @namespace-ignore | @recursive | parent::tpl:element-create-as-parent/@future"/>
            <xsl:attribute name="flat">
                <xsl:choose>
                    <xsl:when test="@flat='true'">TRUE</xsl:when>
                    <xsl:when test="@flat='false'">FALSE</xsl:when>
                    <xsl:when test="not(@flat)">UNSET</xsl:when>
                </xsl:choose>
            </xsl:attribute>
            <xsl:apply-templates select="." mode="IGNORE"/>

            <!-- check -->
            <xsl:for-each select="@out-eval">
                <xsl:if test="../@out">
                    <xsl:message terminate="yes">[BAD TEMPLATE] Not allowed to set both @out and @out-eval on the same node</xsl:message>
                </xsl:if>
            </xsl:for-each>
            <xsl:for-each select="tpl:element-create[@as='preceding-sibling']">
                <xsl:if test="not(following-sibling::tpl:*[1][self::tpl:element|self::tpl:element-ignore|self::tpl:element-create-as-parent])">
                    <xsl:message terminate="yes">[BAD TEMPLATE] Missing relative element for &lt;create-element as="preceding-sibling"&gt;</xsl:message>
                </xsl:if>
            </xsl:for-each>
            <xsl:for-each select="tpl:element-create[@as='following-sibling']">
                <xsl:if test="not(preceding-sibling::tpl:*[1][self::tpl:element|self::tpl:element-ignore|self::tpl:element-create-as-parent])">
                    <xsl:message terminate="yes">[BAD TEMPLATE] Missing relative element for &lt;create-element as="following-sibilng"&gt;</xsl:message>
                </xsl:if>
            </xsl:for-each>

            <!-- variables -->
            <xsl:for-each select="tpl:set">
                <var name="{@variable}"><xsl:copy-of select="@index | text()"/></var>
            </xsl:for-each>

            <!-- in -->
            <xsl:for-each select="tpl:attribute | tpl:attribute-ignore | tpl:element-create-as-parent/tpl:attribute | tpl:element-create-as-parent/tpl:attribute-ignore">
                <a><xsl:apply-templates select="." mode="IGNORE"/><xsl:copy-of select="@in | @out | @if | text()"/></a>
            </xsl:for-each>
            <xsl:apply-templates select="tpl:element | tpl:element-ignore | tpl:element-create-as-parent/tpl:element | tpl:element-create-as-parent/tpl:element-ignore"/>
            <xsl:for-each select="tpl:text | tpl:text-ignore | tpl:element-create-as-parent/tpl:text | tpl:element-create-as-parent/tpl:text-ignore">
                <t><xsl:apply-templates select="." mode="IGNORE"/><xsl:copy-of select="@if | text()"/></t>
            </xsl:for-each>
            <xsl:for-each select="tpl:comment | tpl:comment-ignore | tpl:element-create-as-parent/tpl:comment | tpl:element-create-as-parent/tpl:comment-ignore">
                <c><xsl:apply-templates select="." mode="IGNORE"/><xsl:copy-of select="@if | text()"/></c>
            </xsl:for-each>

            <!-- out -->
            <xsl:for-each select="tpl:attribute-create">
                <create-attribute><xsl:copy-of select="@out | @if | text()"/></create-attribute>
            </xsl:for-each>
            <xsl:for-each select="(parent::tpl:element-create-as-parent/preceding-sibling::tpl:* | preceding-sibling::tpl:*[parent::tpl:element|parent::tpl:element-ignore])
                                  [last()][self::tpl:element-create/@as='preceding-sibling']">
                <create-preceding-sibling><xsl:copy-of select="@if | text()"/></create-preceding-sibling>
            </xsl:for-each>
            <xsl:for-each select="tpl:element-create[@as='first-child']">
                <create-first-child><xsl:copy-of select="@if | text()"/></create-first-child>
            </xsl:for-each>
            <xsl:for-each select="parent::tpl:element-create-as-parent">
                <create-parent>
                    <xsl:if test="not(@group-by)"><xsl:attribute name="group-by">name()</xsl:attribute></xsl:if>
                    <xsl:copy-of select="@out | @if | @attributes | @group-by | text()"/>
                </create-parent>
            </xsl:for-each>
            <xsl:for-each select="tpl:element-create[@as='last-child' or not(@as)]">
                <create-last-child><xsl:copy-of select="@if | text()"/></create-last-child>
            </xsl:for-each>
            <xsl:for-each select="(parent::tpl:element-create-as-parent/following-sibling::tpl:* | following-sibling::tpl:*[parent::tpl:element|parent::tpl:element-ignore])
                                  [1][self::tpl:element-create/@as='following-sibling']">
                <create-following-sibling><xsl:copy-of select="@if | text()"/></create-following-sibling>
            </xsl:for-each>
            <xsl:for-each select="tpl:text-create">
                <create-text><xsl:copy-of select="@if | text()"/></create-text>
            </xsl:for-each>
            <xsl:for-each select="tpl:comment-create">
                <create-comment><xsl:copy-of select="@if | text()"/></create-comment>
            </xsl:for-each>
        </e>
    </xsl:template>

    <xsl:template match="tpl:element|tpl:element-ignore | tpl:attribute|tpl:attribute-ignore" mode="IGNORE">
        <!-- may add attribute @ignore -->
        <xsl:if test="contains(local-name(), '-ignore')">
            <xsl:attribute name="ignore">true</xsl:attribute>
        </xsl:if>
    </xsl:template>
    <xsl:template match="tpl:text|tpl:text-ignore | tpl:comment|tpl:comment-ignore" mode="IGNORE">
        <!-- may add attribute @ignore -->
        <xsl:if test="contains(local-name(), '-ignore')">
            <xsl:attribute name="ignore">true</xsl:attribute>
        </xsl:if>

        <!-- may add attribute @create-parent -->
        <xsl:for-each select="parent::tpl:element-create-as-parent">
            <xsl:attribute name="create-parent"><xsl:value-of select="@out"/></xsl:attribute>
            <xsl:if test="@group-by | @attributes">
                <xsl:message terminate="yes">Attributes @group-by and @attributes are supported only with elements</xsl:message>
            </xsl:if>
        </xsl:for-each>
    </xsl:template>
</xsl:stylesheet>